;;; Russian spelling data
(nconc emacspeak-char-to-phonetic-table
       '(
         ("Á" . "ÁÎÎÁ")
         ("Â" . "ÂÏÒÉÓ")
         ("×" . "×ÁÓÑ")
         ("Ç" . "ÇÏÛÁ")
         ("Ä" . "ÄÉÍÁ")
         ("Å" . "ÅÌÅÎÁ")
         ("£" . "£ÌËÁ")
         ("Ö" . "ÖÏÒÁ")
         ("Ú" . "ÚÑÍÁ")
         ("É" . "É×ÁÎ")
         ("Ê" . "É×ÁÎ ËÒÁÔËÉÊ")
         ("Ë" . "ËÉÌÏ×ÁÔÔ")
         ("Ì" . "ÌÅ×")
         ("Í" . "ÍÁÛÁ")
         ("Î" . "ÎÉÎÁ")
         ("Ï" . "ÏÌØÇÁ")
         ("Ð" . "ÐÁ×ÅÌ")
         ("Ò" . "ÒÏÍÁ")
         ("Ó" . "ÓÏÎÑ")
         ("Ô" . "ÔÁÎÑ")
         ("Õ" . "ÕÌØÑÎÁ")
         ("Æ" . "ÆÅÄÑ")
         ("È" . "ÈÒÅÎ")
         ("Ã" . "ÃÅÎÔÒ")
         ("Þ" . "ÞÁÛÁ")
         ("Û" . "ÛÕÒÁ")
         ("Ý" . "ÝÕËÁ")
         ("ß" . "Ô×ÅÒÄÙÊ ÚÎÁË")
         ("Ù" . "Ù")
         ("Ø" . "ÍÑÇËÉÊ ÚÎÁË")
         ("Ü" . "ÜÄÉË")
         ("À" . "ÀÒÁ")
         ("Ñ" . "ÑÛÁ")
         ("á" . "ÁÎÎÁ ÂÏÌØÛÁÑ")
         ("â" . "ÂÏÒÉÓ ÂÏÌØÛÏÊ")
         ("÷" . "×ÁÓÑ ÂÏÌØÛÏÊ")
         ("ç" . "ÇÏÛÁ ÂÏÌØÛÏÊ")
         ("ä" . "ÄÉÍÁ ÂÏÌØÛÏÊ")
         ("å" . "ÅÌÅÎÁ ÂÏÌØÛÁÑ")
         ("³" . "£ÌËÁ ÂÏÌØÛÁÑ")
         ("ö" . "ÖÏÒÁ ÂÏÌØÛÏÊ")
         ("ú" . "ÚÑÍÁ ÂÏÌØÛÏÊ")
         ("é" . "É×ÁÎ ÂÏÌØÛÏÊ")
         ("ê" . "É×ÁÎ ËÒÁÔËÉÊ ÂÏÌØÛÏÊ")
         ("ë" . "ËÉÌÏ×ÁÔÔ ÂÏÌØÛÏÊ")
         ("ì" . "ÌÅ× ÂÏÌØÛÏÊ")
         ("í" . "ÍÁÛÁ ÂÏÌØÛÁÑ")
         ("î" . "ÎÉÎÁ ÂÏÌØÛÁÑ")
         ("ï" . "ÏÌØÇÁ ÂÏÌØÛÁÑ")
         ("ð" . "ÐÁ×ÅÌ ÂÏÌØÛÏÊ")
         ("ò" . "ÒÏÍÁ ÂÏÌØÛÏÊ")
         ("ó" . "ÓÏÎÑ ÂÏÌØÛÁÑ")
         ("ô" . "ÔÁÎÑ ÂÏÌØÛÁÑ")
         ("õ" . "ÕÌØÑÎÁ ÂÏÌØÛÁÑ")
         ("æ" . "ÆÅÄÑ ÂÏÌØÛÏÊ")
         ("è" . "ÈÒÅÎ ÂÏÌØÛÏÊ")
         ("ã" . "ÃÅÎÔÒ ÂÏÌØÛÏÊ")
         ("þ" . "ÞÁÛÁ ÂÏÌØÛÁÑ")
         ("û" . "ÛÕÒÁ ÂÏÌØÛÏÊ")
         ("ý" . "ÝÕËÁ ÂÏÌØÛÁÑ")
         ("ÿ" . "Ô×ÅÒÄÙÊ ÚÎÁË ÂÏÌØÛÏÊ")
         ("ù" . "Ù ÂÏÌØÛÁÑ")
         ("ø" . "ÍÑÇËÉÊ ÚÎÁË ÂÏÌØÛÏÊ")
         ("ü" . "ÜÄÉË ÂÏÌØÛÏÊ")
         ("à" . "ÀÒÁ ÂÏÌØÛÏÊ")
         ("ñ" . "ÑÛÁ ÂÏÌØÛÏÊ")
         ))
